"""
serializer_to_string

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class serializer_to_string(Operator):
    r"""Take any input and serialize them in a string.


    Parameters
    ----------
    stream_type: int
        0 for string (default), 1 for binary, 2 for binary with chunked output (the output string will be returned in several chunks to prevent string memory overflows).
    any_input1: Any
        any input
    any_input2: Any
        any input

    Returns
    -------
    nof_chunks: int
        Number of chunks when mode passed to input pin(-1) = 2.
    serialized_string1: str
    serialized_string2: str

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.serializer_to_string()

    >>> # Make input connections
    >>> my_stream_type = int()
    >>> op.inputs.stream_type.connect(my_stream_type)
    >>> my_any_input1 = dpf.Any()
    >>> op.inputs.any_input1.connect(my_any_input1)
    >>> my_any_input2 = dpf.Any()
    >>> op.inputs.any_input2.connect(my_any_input2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.serializer_to_string(
    ...     stream_type=my_stream_type,
    ...     any_input1=my_any_input1,
    ...     any_input2=my_any_input2,
    ... )

    >>> # Get output data
    >>> result_nof_chunks = op.outputs.nof_chunks()
    >>> result_serialized_string1 = op.outputs.serialized_string1()
    >>> result_serialized_string2 = op.outputs.serialized_string2()
    """

    def __init__(
        self,
        stream_type=None,
        any_input1=None,
        any_input2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="serializer_to_string", config=config, server=server)
        self._inputs = InputsSerializerToString(self)
        self._outputs = OutputsSerializerToString(self)
        if stream_type is not None:
            self.inputs.stream_type.connect(stream_type)
        if any_input1 is not None:
            self.inputs.any_input1.connect(any_input1)
        if any_input2 is not None:
            self.inputs.any_input2.connect(any_input2)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Take any input and serialize them in a string.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                -1: PinSpecification(
                    name="stream_type",
                    type_names=["int32"],
                    optional=False,
                    document=r"""0 for string (default), 1 for binary, 2 for binary with chunked output (the output string will be returned in several chunks to prevent string memory overflows).""",
                ),
                1: PinSpecification(
                    name="any_input",
                    type_names=["any"],
                    optional=False,
                    document=r"""any input""",
                ),
                2: PinSpecification(
                    name="any_input",
                    type_names=["any"],
                    optional=False,
                    document=r"""any input""",
                ),
            },
            map_output_pin_spec={
                -1: PinSpecification(
                    name="nof_chunks",
                    type_names=["int32"],
                    optional=False,
                    document=r"""Number of chunks when mode passed to input pin(-1) = 2.""",
                ),
                0: PinSpecification(
                    name="serialized_string1",
                    type_names=["string"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="serialized_string2",
                    type_names=["string"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="serializer_to_string", server=server)

    @property
    def inputs(self) -> InputsSerializerToString:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsSerializerToString.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsSerializerToString:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsSerializerToString.
        """
        return super().outputs


class InputsSerializerToString(_Inputs):
    """Intermediate class used to connect user inputs to
    serializer_to_string operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.serializer_to_string()
    >>> my_stream_type = int()
    >>> op.inputs.stream_type.connect(my_stream_type)
    >>> my_any_input1 = dpf.Any()
    >>> op.inputs.any_input1.connect(my_any_input1)
    >>> my_any_input2 = dpf.Any()
    >>> op.inputs.any_input2.connect(my_any_input2)
    """

    def __init__(self, op: Operator):
        super().__init__(serializer_to_string._spec().inputs, op)
        self._stream_type = Input(
            serializer_to_string._spec().input_pin(-1), -1, op, -1
        )
        self._inputs.append(self._stream_type)
        self._any_input1 = Input(serializer_to_string._spec().input_pin(1), 1, op, 0)
        self._inputs.append(self._any_input1)
        self._any_input2 = Input(serializer_to_string._spec().input_pin(2), 2, op, 1)
        self._inputs.append(self._any_input2)

    @property
    def stream_type(self) -> Input:
        r"""Allows to connect stream_type input to the operator.

        0 for string (default), 1 for binary, 2 for binary with chunked output (the output string will be returned in several chunks to prevent string memory overflows).

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> op.inputs.stream_type.connect(my_stream_type)
        >>> # or
        >>> op.inputs.stream_type(my_stream_type)
        """
        return self._stream_type

    @property
    def any_input1(self) -> Input:
        r"""Allows to connect any_input1 input to the operator.

        any input

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> op.inputs.any_input1.connect(my_any_input1)
        >>> # or
        >>> op.inputs.any_input1(my_any_input1)
        """
        return self._any_input1

    @property
    def any_input2(self) -> Input:
        r"""Allows to connect any_input2 input to the operator.

        any input

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> op.inputs.any_input2.connect(my_any_input2)
        >>> # or
        >>> op.inputs.any_input2(my_any_input2)
        """
        return self._any_input2


class OutputsSerializerToString(_Outputs):
    """Intermediate class used to get outputs from
    serializer_to_string operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.serializer_to_string()
    >>> # Connect inputs : op.inputs. ...
    >>> result_nof_chunks = op.outputs.nof_chunks()
    >>> result_serialized_string1 = op.outputs.serialized_string1()
    >>> result_serialized_string2 = op.outputs.serialized_string2()
    """

    def __init__(self, op: Operator):
        super().__init__(serializer_to_string._spec().outputs, op)
        self._nof_chunks = Output(serializer_to_string._spec().output_pin(-1), -1, op)
        self._outputs.append(self._nof_chunks)
        self._serialized_string1 = Output(
            serializer_to_string._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._serialized_string1)
        self._serialized_string2 = Output(
            serializer_to_string._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._serialized_string2)

    @property
    def nof_chunks(self) -> Output:
        r"""Allows to get nof_chunks output of the operator

        Number of chunks when mode passed to input pin(-1) = 2.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> # Get the output from op.outputs. ...
        >>> result_nof_chunks = op.outputs.nof_chunks()
        """
        return self._nof_chunks

    @property
    def serialized_string1(self) -> Output:
        r"""Allows to get serialized_string1 output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> # Get the output from op.outputs. ...
        >>> result_serialized_string1 = op.outputs.serialized_string1()
        """
        return self._serialized_string1

    @property
    def serialized_string2(self) -> Output:
        r"""Allows to get serialized_string2 output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serializer_to_string()
        >>> # Get the output from op.outputs. ...
        >>> result_serialized_string2 = op.outputs.serialized_string2()
        """
        return self._serialized_string2
